#!/usr/bin/python
#
# Copyright (C) 2012 Intel Corporation
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Authors:
#              Tang, Shaofeng <shaofeng.tang@intel.com>

import ConfigParser
from xml.etree import ElementTree
import xml.etree.ElementTree as etree
import re
import os,os.path
import zipfile
import subprocess
import time
import shutil
import platform
import tempfile
from commodule.killall import killall

CONFIG_FILE = "/opt/tct/tizen_3.0/shell/CONFIG"
VERSION_FILE = "/opt/tct/tizen_3.0/shell/VERSION"
parser = ConfigParser.ConfigParser()
parser.read(CONFIG_FILE)
vparser = ConfigParser.ConfigParser()
vparser.read(VERSION_FILE)


class Constants:

     TCT_VERSION = vparser.get('public_version', 'version')

     TEST_SUITE_DIR  = parser.get('TCTSHELL', 'TEST_SUITE_DIR')

     WRT_LAUNCHR_CMD = parser.get('TCTSHELL', 'WRT_LAUNCHR_CMD')

     RELEASED_SUITE_LOCATION = TEST_SUITE_DIR + "%s"

     TCT_SHELL_HOME = parser.get('TCTSHELL', 'TCT_SHELL_HOME')

     TEMP_UNPASSED_XML_FOLDER = TCT_SHELL_HOME + "/tmp/"

     TEMP_RESULT_XML_FOLDER = TCT_SHELL_HOME + "/result/"

     if not TEMP_UNPASSED_XML_FOLDER.endswith('/'):
            TEMP_UNPASSED_XML_FOLDER = TEMP_UNPASSED_XML_FOLDER + "/"
     
     RERUN_OUTPUT = " -o " + TEMP_UNPASSED_XML_FOLDER + "result.xml"

     CAPABILITY_SUITE_NAME = "capability-tests"

     TCT_PLAN_FOLDER = parser.get('TCTSHELL', 'TCT_PLAN_FOLDER')

     STYLE_FOLDER = TCT_SHELL_HOME + "/style/*"

     HOST_ZIP_PATH = TCT_SHELL_HOME + "/tmp"

     HOST_TESTS_FILE = TCT_SHELL_HOME + "/tmp/opt/%s/tests.xml"

     CAPABILITY_PATH = TCT_SHELL_HOME + "/tmp/capability.xml"

     BUILD_INFO_PATH = TCT_SHELL_HOME + "/tmp/buildinfo.xml"

     DEVICE_CAPABILITY_PATH = parser.get('Device', 'DEVICE_CAPABILITY_PATH')

     DEVICE_BUILD_INFO_PATH = parser.get('Device', 'DEVICE_BUILD_INFO_PATH')

     DEVICE_DEFAULT_CONF = parser.get('Device', 'DEVICE_DEFAULT_CONF_PATH')
     
     DEVICE_DEFAULT_JSON_CONF = parser.get('Device', 'DEVICE_DEFAULT_ISON_CONF_PATH')

     DEVICE_HEALTH_CHECK_CMD = "python /opt/tct/tizen_3.0/scripts/tct-config-device.py --check"

     PRE_CONFIGURE_FILE_PATH = "/opt/tct/tizen_3.0/shell/tmp/pre_configure.conf"
     
     PRE_CONFIGURE_ISON_FILE_PATH = "/opt/tct/tizen_3.0/shell/tmp/preconfigure.json"

     DEVICE_CONFIGURE_FILE_PATH = parser.get('Device','DEVICE_PRECONFIGURE_FILE_PATH')

     TCT_TESTCONFIG_PATH = "/home/app/content/tct/opt/tct-testconfig/tests.xml"

     HEALTH_CHECK_FILE_PATH = "/opt/tct/tizen_3.0/scripts/healthcheck.ini"

     #PARAM Option
     LITE_CMD = "/opt/tct/tizen_3.0/testkit-lite"
     XMLFILE_PREFF = "-f "
     ONLY_AUTO_CASES = "-A"
     ONLY_MANUAL_CASES = "-M"
     
     #SDB Command
     SDB_SHELL_TIMEOUT = 90
     SDB_PUSH = "sdb %s push"
     SDB_SHELL = "sdb %s shell"
     SDB_DEVICES = "sdb devices"
     SDB_PULL = "sdb %s pull"
     SDB_CAT_CONFIG = " cat %s"
     SDB_GET_DLOG_PRO = SDB_SHELL + " dlogctrl get platformlog"

     
     #DEVICE
     DEVICE_TMP_FOLDER = parser.get('Device', 'DEVICE_TMP_FOLDER')
     DEVICE_ZIP_PATH = "/home/app/content/tct"
     DEVICE_TESTS_FILE = parser.get('Device', 'DEVICE_TESTS_FILE') + " "

     #Testkit-Lite
     LITE_HOME = parser.get('TESTKIT_LITE', 'LITE_HOME')

     INSTALL_RPM       = "rpm -i --nodeps /tmp/%s"
     UNINSTALL_RPM     = "rpm -e --nodeps %s"
     REMOVE_RPM        = "rm -f /tmp/%s"
     INSTALL_ZIP = "inst.py"
     APP_USER_ENV = "su - app -c 'export DBUS_SESSION_BUS_ADDRESS=unix:path=/run/user/5000/dbus/user_bus_socket;%s'"

     #Plan Generator
     ALL_PROFILES = ['ivi', 'mobile', 'wearable']
     DEFAULT_MATCH_REGEX = "*.zip"
     DEFAULT_UNMATCH_REGEX = None
     DEFAULT_PLAN_NAME = "Full_test"
     DEFAULT_PROFILE_NAME = "ivi"
     DEFAULT_EXECUTE_TYPE = "All"
     DEFAULT_PLAN_FILE = "tct_plan"

     REGEX_FOR_SPLIT_PKG_NAME = "-(\d+)\.(\d+)\.(\d+)"

     #Running Mode
     RUNNING_MODE_PLAN = "plan"
     RUNNING_MODE_RESULT = "result"
     RUNNING_MODE_SUITES = "suites"
     INSTALL_MODE_SUITES = "instsuites"
     UNINSTALL_MODE_SUITES = "uninstsuites"

     @staticmethod
     def unzip_package(filepath):
         path = os.path.expanduser("~")
         os.chdir(path)
         if os.path.exists(Constants.DEFAULT_PLAN_FILE):
             shutil.rmtree(Constants.DEFAULT_PLAN_FILE)
         os.makedirs(Constants.DEFAULT_PLAN_FILE)
         f = zipfile.ZipFile(filepath,'r')
         #print "new path: ", new_path
         for file in f.namelist():
             f.extract(file,path + "/"+Constants.DEFAULT_PLAN_FILE)

     @staticmethod
     def clean_unzip_file():
         os.chdir(os.path.expanduser("~"))
         shutil.rmtree(Constants.DEFAULT_PLAN_FILE)
         
     @staticmethod
     def add_right_enbrace(param):
         param = param.rstrip()
         param += "\""
         return param

     @staticmethod
     def indent(elem, level=0):
         i = "\n" + level*"  "
         if len(elem):
             if not elem.text or not elem.text.strip():
                 elem.text = i + "  "
             if not elem.tail or not elem.tail.strip():
                 elem.tail = i
             for elem in elem:
                 Constants.indent(elem, level+1)
             if not elem.tail or not elem.tail.strip():
                 elem.tail = i
         else:
             if level and (not elem.tail or not elem.tail.strip()):
                 elem.tail = i

     @staticmethod
     def copy_style_in_result_folder(result_folder,wrapper):
         os.system("cp -r " + Constants.STYLE_FOLDER + " '" + result_folder + "'")
         resultfile = result_folder + "result.xml"
         if wrapper.is_output_defined():
             resultfile = wrapper.get_output_file_name()
         print "Going to open the result report with FireFox, if Firefox is not available on your machine, please copy the folder '%s' to a Firefox machine." % (result_folder + "summary.xml")
         print "If you want to rerun,please run cmd: /opt/tct/tizen_3.0/tct-shell --rerun-fail '%s'" % (result_folder + "result.xml") + wrapper.is_manual_or_all_param()
         os.system("firefox '%s'summary.xml &" % result_folder)
    
     @staticmethod
     def execute_sdb_command(command):
         time_out = 0
         result = None
         exit_code = None
         out_file = tempfile.SpooledTemporaryFile(bufsize=512*1024)
         fileno = out_file.fileno()
         proc = subprocess.Popen(command,
                            shell=True,
                            stdout=fileno,
                            stderr=fileno)
         while time_out < Constants.SDB_SHELL_TIMEOUT:
             exit_code = proc.poll()
             if exit_code is not None:break
             time_out += 0.2
             time.sleep(0.2)

         if exit_code is None:
             print "sdb time out,kill subprocess..."
             killall(proc.pid)
         else:
             out_file.seek(0)
             result = out_file

         return exit_code, result

     @staticmethod
     def cmd_wrapper(cmd, launcher):
         if launcher is not None and launcher.find('xwalk') >= 0:
             return Constants.APP_USER_ENV % (cmd)
         else:
             return cmd   

     @staticmethod
     def get_suite_items(filesname,filepath):
         Constants.unzip_package(filepath)
         m = re.split(Constants.REGEX_FOR_SPLIT_PKG_NAME, filesname)
         name = m[0]
         test_file = os.path.expanduser("~") + "/"+Constants.DEFAULT_PLAN_FILE+"/opt/" + name + "/tests.xml"
         if not os.path.exists(test_file):
             print("Can't find the test file...")
             return 0,0,""
         return Constants.__read_test_xml(test_file)

     @staticmethod
     def __read_test_xml(test_file):
         autocnt = manualcnt = 0
         test_xml_temp = etree.parse(test_file)
 
         for test_xml_temp_suite in test_xml_temp.getiterator('suite'):
             suite_name = test_xml_temp_suite.get('name')
             launcher = test_xml_temp_suite.get('launcher')
             if launcher is None:
                 launcher = ""
             autocnt = manualcnt = 0
             for tset in test_xml_temp_suite.getiterator('set'):
                 set_autocnt, set_manualcnt = Constants.__get_set_casecnt(tset)
                 autocnt += set_autocnt
                 manualcnt += set_manualcnt
         return autocnt, manualcnt, launcher

     @staticmethod
     def __get_set_casecnt(tset):
          set_autocnt = set_manualcnt = 0
          for tcase in tset.getiterator('testcase'):
              exetype = tcase.get('execution_type')
              if exetype == "auto":
                  set_autocnt += 1
              elif exetype == "manual":
                  set_manualcnt += 1
          return set_autocnt, set_manualcnt
