#!/usr/bin/python
#
# Copyright (C) 2012 Intel Corporation
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Authors:
#              Tang, Shaofeng <shaofeng.tang@intel.com>

from constants import Constants
from xml.etree import ElementTree
import xml.etree.ElementTree as etree
import os,os.path
import zipfile
import glob
import re

class SuitePackage:
    def __init__(self, profile, file_name, command,auto_count,manual_count,launcher):
        self.file = file_name
        self.profile = profile
        m = re.split(Constants.REGEX_FOR_SPLIT_PKG_NAME, file_name)
        self.name = m[0]
        self.auto_count = auto_count
        self.manual_count = manual_count
        self.launcher = launcher
        self.total_count = auto_count + manual_count
        self.command = command + " %s" % self.name

    def to_xml(self):
        suite = ElementTree.Element('suite')
        suite.set('name', self.name)
        suite.set('launcher', self.launcher)
        auto = ElementTree.Element('auto_tcn')
        auto.text = str(self.auto_count)
        suite.append(auto)
        manual = ElementTree.Element('manual_tcn')
        manual.text = str(self.manual_count)
        suite.append(manual)
        total = ElementTree.Element('total_tcn')
        total.text = str(self.total_count)
        suite.append(total)
        rpm_name = ElementTree.Element('pkg_name')
        rpm_name.text = self.profile + '/' + self.file
        suite.append(rpm_name)
        return suite

class PlanRunner:

    def __init__(self):
        self.suites = {}
        self.plan_name = Constants.DEFAULT_PLAN_NAME
        self.profile = Constants.DEFAULT_PROFILE_NAME
        self.execute_type = Constants.DEFAULT_EXECUTE_TYPE

    def load_local_repo(self, path, match, command, unmatch, plan_name, profile, execute_type):
        try:
            self.plan_name = plan_name
            self.profile = profile
            self.execute_type = execute_type
            repos = ['common', profile]
            for tp in repos:
                repo = '/'.join([path, tp])
                os.chdir(repo)
                print "match: %s" % match
                print "unmatch: %s" % unmatch
                for files in glob.glob(match):
                    print "File name: %s" % str(files)
                    if (unmatch is not None) and (re.search(unmatch, str(files))):
                        print "File %s is skipped with unmatch regex %s" % (str(files), unmatch)
                    else:
                        auto,manual,launcher = Constants.get_suite_items(files, repo + '/' + str(files))
                        suite = SuitePackage(tp, files, command, auto, manual, launcher)
                        self.suites[suite.name] = suite
                        Constants.clean_unzip_file()
        except Exception, e:
            print "[ Error happen when reading the local repository, error: %s ]\n" % e

    def to_xml(self, xml_name):
        print "generating plan to %s" % xml_name
        root = ElementTree.Element('ns3:testplan')
        root.set('name', self.plan_name)
        root.set('profile', self.profile)
        root.set('xmlns','')
        root.set('xmlns:ns3', 'http://www.example.org/plan/')
        element = ElementTree.Element('execute_type')
        element.text = self.execute_type
        root.append(element)
        for suite_name in self.suites:
            suite = self.suites[suite_name]
            root.append(suite.to_xml())
        Constants.indent(root)
        tree = ElementTree.ElementTree()
        tree._setroot(root)
        tree.write(xml_name, encoding="utf-8")
