#!/usr/bin/python

"""Usage:
         tct-config-device.py [--install [--deviceid <serial_no>]]
         tct-config-device.py --purge [--deviceid <serial_no>]
         tct-config-device.py --check [--deviceid <serial_no>][--procid <process_name>]
         tct-config-device.py --bhtest <profile>

Options:
  -h, --help         show this help message and exit
  --install          Install tct resources to TIZEN device,it can be omitted
  --purge            Clean up tct resources from TIZEN device
  --check            Check test resource on device
  --bhtest           Install tct behavior test
  --deviceid         Set the device serial number
  --procid           Set the check process name

"""
import ConfigParser
import os
import sys
import subprocess
import time
import platform
from optparse import OptionParser

JOIN = os.path.join
EXISTS = os.path.exists
DIRNAME = os.path.dirname
BASENAME = os.path.basename
ABSPATH = os.path.abspath
SPLIT = os.path.split

CURENT_DIR = SPLIT(DIRNAME(ABSPATH(__file__)))[0]

TOOLS_DIR = JOIN(CURENT_DIR, "tools")
RESRC_DIR = JOIN(CURENT_DIR, "resource")
PKGS_DIR = JOIN(CURENT_DIR, "package")
SDB_SERNO = "sdb"
EXE_MODE = "--install"
WRT_INSTALLER = "pkgcmd"
DEV_ARCH = "arm"
SDB_SHELL_TIMEOUT = 90


def LS(root, file_name):
    import glob
    file_path = JOIN(root, file_name) + '*'
    files = glob.iglob(r'' + file_path + '')
    for f in files:
        return BASENAME(f)


def execute_sdb_command(command, maxtime=SDB_SHELL_TIMEOUT):
    proc = subprocess.Popen(command,
                            shell=True,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
    time_out = 0
    result = []
    exit_code = None
    while time_out < maxtime:
        exit_code = proc.poll()
        if exit_code is not None:
            break
        time_out += 0.5
        time.sleep(0.5)
    if exit_code is None:
        print "sdb time out,kill subprocess..."
        killall(proc.pid)
    else:
        result = proc.stdout.readlines() or proc.stderr.readlines()

    return exit_code, result

import signal
import re
import ctypes


def killall(ppid):
    """Kill all children process by parent process ID"""
    os_ver = platform.system()
    try:
        if os_ver == "Linux" or os_ver == "Darwin":
            ppid = str(ppid)
            pidgrp = []

            def getchildpids(ppid):
                """Return a list of children process"""
                command = "ps -ef | awk '{if ($3 == %s) print $2;}'" % str(
                    ppid)
                pids = os.popen(command).read()
                pids = pids.split()
                return pids

            pidgrp.extend(getchildpids(ppid))
            for pid in pidgrp:
                pidgrp.extend(getchildpids(pid))
            # Insert self process ID to PID group list
            pidgrp.insert(0, ppid)
            while len(pidgrp) > 0:
                pid = pidgrp.pop()
                try:
                    os.kill(int(pid), signal.SIGKILL)
                except OSError, error:
                    pattern = re.compile('No such process')
                    match = pattern.search(str(error))
                    if not match:
                        print "[ Error: fail to kill pid: %s," \
                            " error: %s ]\n" % (int(pid), error)
        # kill for windows platform
        else:
            kernel32 = ctypes.windll.kernel32
            handle = kernel32.OpenProcess(1, 0, int(ppid))
            kernel32.TerminateProcess(handle, 0)
    except OSError, error:
        pattern = re.compile('No such process')
        match = pattern.search(str(error))
        if not match:
            print "[ Error: fail to kill pid: %s, error: %s ]\n" \
                  % (int(ppid), error)
    return None


def sdb_root_on():
    print "set sdb root on. Please wait..."
    cmd = SDB_SERNO + ' root on'
    result = os.system(cmd)
    if result:
        print "Error: set root mode for Tizen device failed.\n\
    Please confirm your device connected to machine and flashed with proper TIZEN image\n\
    And then re-run tct-config-device script"
        sys.exit(1)

def get_device_arch():
    global DEV_ARCH
    ARCHCMD = SDB_SERNO + " shell \"uname -m\""
    result = execute_sdb_command(ARCHCMD)[1]
    if len(result) > 0:
        arch_str = result[0].strip('\r\n')
        print "Get device cpu_arch type: %s" % arch_str
        if arch_str.startswith('arm'):
            DEV_ARCH = "arm"
        elif arch_str.startswith("i686"):
            DEV_ARCH = "ia"
        else:
            print "Unsupported arch type: %s" % arch_str
            return False
        return True
    else:
        print "Failed to get device cpu_arch type"
        return False  


def kill_process(process_name):
    PRID = SDB_SERNO + " shell \"ps aux | grep " + \
        process_name + " | grep -v grep | awk '{print $2}' \""
    result = execute_sdb_command(PRID)[1]
    if len(result) > 0:
        result = result[0].strip()
        kill_cmd = SDB_SERNO + ' shell \"kill -9 ' + \
            result + ' > /dev/null 2>&1 \"'
        os.system(kill_cmd)
        print "%s process has been killed" % process_name
    else:
        print "No process of %s activated" % process_name


def remove_tinyweb():
    os.system(
        SDB_SERNO + ' shell "rm -f /opt/home/developer/server.pem > /dev/null 2>&1 "')
    os.system(
        SDB_SERNO + ' shell "rm -f /opt/home/developer/tinyweb > /dev/null 2>&1 "')
    os.system(
        SDB_SERNO + ' shell "rm -f /opt/home/developer/cgi-getcookie > /dev/null 2>&1 "')
    os.system(
        SDB_SERNO + ' shell "rm -f /opt/home/developer/cgi-getfield > /dev/null 2>&1 "')
    os.system(
        SDB_SERNO + ' shell "rm -f /opt/home/developer/libmongoose.so > /dev/null 2>&1 "')
    os.system(
        SDB_SERNO + ' shell "rm -f /opt/home/developer/echo.so > /dev/null 2>&1 "')


def remove_stub():
    os.system(
        SDB_SERNO + ' shell "rm -f /opt/home/developer/testkit-stub > /dev/null 2>&1 "')

def uninstall_testconfig():
    print "Uninstall testconfig. Please wait..."
    RPMPKG = LS(RESRC_DIR, "tct-testconfig")
    path = RESRC_DIR + os.sep + 'opt' + os.sep + 'tct-testconfig' + os.sep + 'inst.py'
    cur_path = os.getcwd()
    if not os.path.exists(path):
        pkgfile = RESRC_DIR + os.sep + RPMPKG
        os.chdir(RESRC_DIR)
        os.system("unzip %s" %pkgfile)
        os.chdir(os.path.dirname(path)) 
        os.system("python %s -u" %path)
        os.chdir(cur_path)
    result = os.system("python %s" %path)

def uninstall_behavior():
    print "Uninstall behavior tool. Please wait..."
    path = TOOLS_DIR + os.sep + 'opt' + os.sep + 'tct-behavior-tests' + os.sep + 'inst.py'
    if not os.path.exists(path):
        RPMPKG = LS(TOOLS_DIR, "tct-behavior-tests")
        os.system("unzip %s -d %s > /dev/null 2>&1" %(RPMPKG, TOOLS_DIR))
    os.system("python %s -u" %path)

def uninstall_widget(wgtname):
    print "Uninstall widget %s. Please wait..." % wgtname
    if platform.system() == 'Linux':
        if cmp(wgtname, 'getCap') != 0:
            cmd = SDB_SERNO + " shell  app_launcher -l | grep " + wgtname + \
                "| awk '{ print $(NF-1) }' | sed 's/[[:space:]]*$//' "
        else:
            cmd = SDB_SERNO + " shell su - app -c \"app_launcher -l | grep tct-getcapabilities" \
                "| awk '{ print $(NF-1) }' | sed 's/[[:space:]]*$//'\" "

    else:
        cmd = SDB_SERNO + " shell \" app_launcher -l | grep " + wgtname + \
            "| awk '{ print $(NF-1) }' | sed 's/[[:space:]]*$//' \" "
    WGTNAME = execute_sdb_command(cmd)[1]
    if cmp(wgtname,'getCap') == 0:
        WGTNAME = WGTNAME[0].split(' ')
    if len(WGTNAME) > 0:
        WGTNAME = WGTNAME[0].strip()
        print "Try to uninstall the widget %s..." % wgtname
        cmd = SDB_SERNO +  ' shell " pkgcmd -u -q -t wgt -n ' + \
            WGTNAME + '; echo uistl=$? "'
        UNINSTL_info = execute_sdb_command(cmd)[1]
        for line in UNINSTL_info:
            if 'uistl' in line:
                start = line.index('uistl') + 6
                end = line.index('\r')
                UNINSTL = int(line[start:end])
        if UNINSTL:
            print "Uninstall widget %s failed. Please uninstall it manually via following command:\n\
      %s shell\
      pkgcmd -u -q -t wgt -n %s" % (wgtname, SDB_SERNO, WGTNAME)
        else:
            print "Widget %s uninstalled successfully" % wgtname
    else:
        print "Widget %s is not installed. No need to uninstall" % wgtname


def check_resource_dir():
    print "check resource directory. Please wait..."
    if os.path.exists(RESRC_DIR):
        print "The directory resource exists"
    else:
        print "ERROR: The directory resource does not exists. Please copy test resources to %s " % RESRC_DIR
        sys.exit(255)
    os.system(
        SDB_SERNO + ' shell "mkdir -p /home/app/content/tct/ > /dev/null 2>&1 " ')
    os.system(
        SDB_SERNO + ' shell "mkdir -p /home/app/content/tct/Documents/ > /dev/null 2>&1 "')


def check_tinyweb():
    print "Check tinyweb. Please wait..."
    cmd = SDB_SERNO + " shell \"ls /opt/home/developer/ | grep tinyweb\" "
    CHECKTINYWEB = execute_sdb_command(cmd)[1]
    if len(CHECKTINYWEB) > 0:
        launch_tinyweb()
    else:
        print "no tinyweb in device. Now install tinyweb"
        install_tinyweb()


def launch_tinyweb():
    DPATH_cmd = SDB_SERNO + ' shell \"printenv PATH\"'
    DPATH = execute_sdb_command(DPATH_cmd)[1][0].strip()
    launch_cmd = SDB_SERNO + ' shell "env LD_LIBRARY_PATH=/opt/home/developer PATH=' + DPATH + \
        ':/opt/home/developer tinyweb -ssl_certificate /opt/home/developer/server.pem -document_root /home/app/content/tct/ -listening_ports 80,8080,8081,8082,8083,8443s; sleep 3s > /dev/null 2>&1 " '
    execute_sdb_command(launch_cmd, maxtime=8)
    check_tinyweb_cmd = SDB_SERNO + \
        ' shell "ps aux | grep tinyweb | grep -v grep"'
    TINYWEBSTATUS = execute_sdb_command(check_tinyweb_cmd)[1]
    if len(TINYWEBSTATUS) > 0:
        print "Active process tinyweb successfully."
    else:
        print "Active process tinyweb failed."
        sys.exit(255)


def install_tinyweb():
    print "Install tinyweb. Please wait..."
    cmd1 = SDB_SERNO + ' push "' + \
        JOIN(RESRC_DIR, 'tinyweb', DEV_ARCH, 'tinyweb') + '" /opt/home/developer/'
    os.system(cmd1)
    os.system(
        SDB_SERNO + ' shell "chmod a+x /opt/home/developer/tinyweb > /dev/null 2>&1 " ')
    cmd2 = SDB_SERNO + ' push "' + \
        JOIN(RESRC_DIR, 'tinyweb', DEV_ARCH, 'cgi-getcookie') + '" /opt/home/developer/'
    os.system(cmd2)
    os.system(
        SDB_SERNO + ' shell "chmod a+x /opt/home/developer/cgi-getcookie > /dev/null 2>&1 "')
    cmd3 = SDB_SERNO + ' push "' + \
        JOIN(RESRC_DIR, 'tinyweb', DEV_ARCH, 'cgi-getfield') + '" /opt/home/developer/ '
    os.system(cmd3)
    os.system(
        SDB_SERNO + ' shell "chmod a+x /opt/home/developer/cgi-getfield > /dev/null 2>&1 "')
    cmd4 = SDB_SERNO + ' push "' + \
        JOIN(RESRC_DIR, 'tinyweb', DEV_ARCH, 'libmongoose.so') + \
        '" /opt/home/developer/ '
    os.system(cmd4)
    os.system(
        SDB_SERNO + ' shell "chmod 666 /opt/home/developer/libmongoose.so > /dev/null 2>&1 "')
    cmd5 = SDB_SERNO + ' push "' + \
        JOIN(RESRC_DIR, 'tinyweb', DEV_ARCH, 'echo.so') + '" /opt/home/developer/ '
    os.system(cmd5)
    os.system(
        SDB_SERNO + ' shell "chmod 666 /opt/home/developer/echo.so > /dev/null 2>&1 "')
    cmd6 = SDB_SERNO + ' push "' + \
        JOIN(RESRC_DIR, 'tinyweb', DEV_ARCH, 'server.pem') + '" /opt/home/developer/ '
    os.system(cmd6)
    os.system(
        SDB_SERNO + ' shell "chmod 666 /opt/home/developer/server.pem > /dev/null 2>&1 "')
    os.system(
        SDB_SERNO + ' shell "ln -s /usr/lib/libssl.so.1.0.0 /opt/home/developer/libssl.so > /dev/null 2>&1"')
    os.system(
        SDB_SERNO + ' shell "ln -s /usr/lib/libcrypto.so.1.0.0 /opt/home/developer/libcrypto.so > /dev/null 2>&1"')


def check_stub():
    print "Check testkit-stub. Please wait..."
    cmd = SDB_SERNO + " shell \"ls /opt/home/developer/ | grep testkit-stub\" "
    CHECKSTUBA = execute_sdb_command(cmd)[1]
    if len(CHECKSTUBA) > 0:
        print "testkit-stub has been installed on device."
    else:
        install_stub()


def remove_stub():
    os.system(
        SDB_SERNO + ' shell "rm -f /opt/home/developer/testkit-stub > /dev/null 2>&1 "')


def install_stub():
    print "Install testkit-stub. Please wait..."
    cmd1 = SDB_SERNO + ' push "' + \
        JOIN(RESRC_DIR, 'testkit-stub', DEV_ARCH, 'testkit-stub') + \
        '" /opt/home/developer/testkit-stub '
    os.system(cmd1)
    os.system(
        SDB_SERNO + ' shell "chmod +x /opt/home/developer/testkit-stub > /dev/null 2>&1 "')


def check_testconfig():
    CHECKCONFIG_cmd = SDB_SERNO + \
        ' shell "cat /home/app/content/tct/opt/tct-testconfig/tct-testconfig.ini 2>/dev/null | grep tct"'
    CHECKCONFIG = execute_sdb_command(CHECKCONFIG_cmd)[1]
    if len(CHECKCONFIG) > 0:
        print "testconfig has already been installed on device."
    else:
        print "no testconfig in device. Now Install it"
        install_testconfig()

def install_testconfig():
    print "Install testconfig. Please wait..."
    RPMPKG = LS(RESRC_DIR, "tct-testconfig")
    path = RESRC_DIR + os.sep +  'opt' + os.sep + 'tct-testconfig' + os.sep + 'inst.py'
    if not os.path.exists(path):
        pkgfile = RESRC_DIR + os.sep + RPMPKG
        os.chdir(RESRC_DIR)
        cur_path = os.getcwd()
        os.system('unzip %s -d tct-testconfig' %pkgfile)
        time.sleep(2)
        os.system("python %s" %path)
        os.chdir(cur_path)
    CHECKCONFIG_cmd = SDB_SERNO + \
        ' shell "cat /home/app/content/tct/opt/tct-testconfig/tct-testconfig.ini 2>/dev/null | grep tct"'
    CHECKCONFIG = execute_sdb_command(CHECKCONFIG_cmd)[1]
    if len(CHECKCONFIG) > 0:
        print "Install testconfig successfully."
    else:
        print "Failed to install testconfig, please check $RESRC_DIR."
        sys.exit(255)


def install_behavior(profile_name):
    print "Install behavior tool. Please wait..."
    if not profile_name in ['wearable', 'mobile', 'ivi']:
        print "Invalid device profile value %s" % profile_name
        sys.exit(255)
    DIR_PKG = JOIN(PKGS_DIR, profile_name)
    path = TOOLS_DIR + os.sep + 'opt' + os.sep + 'tct-behavior-tests' + os.sep + 'inst.py'
    if not os.path.exists(path):
        RPMPKG = LS(DIR_PKG, "tct-behavior-tests")
        os.system("unzip %s -d %s > /dev/null 2>&1" %(RPMPKG, TOOLS_DIR))
    os.system("python %s" %path)

def install_widget(widget_name):
    print "Install widget %s. Please wait..." % widget_name
    WIDGETPKG = LS(RESRC_DIR, widget_name)
    cmd1 = SDB_SERNO + ' push "' + \
        JOIN(RESRC_DIR, WIDGETPKG) + '" /home/app/content/tct/' + WIDGETPKG

    os.system(cmd1)
    if cmp(WIDGETPKG,'getCap.wgt') != 0: 
        os.system(
            SDB_SERNO + ' shell pkgcmd -i -t wgt -q -p /home/app/content/tct/' + WIDGETPKG)
    else:
        ret = os.system(
            SDB_SERNO + " shell su - app -c 'pkgcmd -i -t wgt -q -p /home/app/content/tct/'" + WIDGETPKG)

def check_getCap():
    print "Check getCap widget. Please wait..."
    WIDGETID_cmd = SDB_SERNO + \
        " shell su - app -c \"app_launcher -l | grep tct-getcapabilities | awk '{ print $NF }' | sed 's/[[:space:]]*$//'\""
    WIDGETID = execute_sdb_command(WIDGETID_cmd)[1]
    print 'debug',WIDGETID 
    if len(WIDGETID) > 0:
        print "widget getCap has already been installed on device."
    else:
        install_widget("getCap")
    launch_widget("getCap")
    time.sleep(5)
    CAPRESULT_cmd = SDB_SERNO + \
        ' shell "cat /home/app/content/Documents/tct/buildinfo.xml 2>/dev/null | grep buildinfos" '
    CAPRESULT = execute_sdb_command(CAPRESULT_cmd)[1]
    if len(CAPRESULT) == 0:
        print "getCap get buildinfo of device failed!"
        sys.exit(255)


def launch_widget(widget_name):
    print "Check widget %s. Please wait..." % widget_name
    if platform.system() == 'Linux':
        if cmp(widget_name,'getCap') !=0:
            cmd = SDB_SERNO + " shell  app_launcher -l | grep " + \
                widget_name + "| awk '{ print $NF }' | sed 's/[[:space:]]*$//'"
        else:
            cmd = SDB_SERNO + " shell su - app -c \"app_launcher -l | grep " + \
                'tct-getcapabilities' + "| awk '{ print $NF }' | sed 's/[[:space:]]*$//'\""

    else:
        cmd = SDB_SERNO + " shell \" app_launcher -l | grep " + widget_name + \
            "| awk '{ print $NF }' | sed 's/[[:space:]]*$//' \" "
    WGTID = execute_sdb_command(cmd)[1]
    
    if cmp(widget_name,'getCap') == 0:
        WGTID = WGTID[0].split(' ')[1]
    #print 'debug', WGTID

    if len(WGTID) > 0:
        print "Try to launch the %s widget..." % widget_name
        if cmp(widget_name, 'getCap') != 0: 
            LAUNCHRESULT_cmd = SDB_SERNO + ' shell \" app_launcher -s ' + \
                WGTID[0].strip() + '\"'
        else:
            LAUNCHRESULT_cmd = SDB_SERNO + ' shell su - app -c  \" app_launcher -s ' + \
                WGTID.strip() + '\"'

        LAUNCHRESULT = execute_sdb_command(LAUNCHRESULT_cmd)[1][0]
        if "launched" in LAUNCHRESULT:
            print "Launch %s successfully" % widget_name
        else:
            print "Launch %s got failed" % widget_name
            sys.exit(255)
    else:
        print "Widget %s cannot be found. Please check whether the widget is installed successfully" % widget_name
        sys.exit(255)

if __name__ == "__main__":

    parser = OptionParser()
    parser.add_option(
        "--install", dest="install", action="store_true", default=False,
        help="Install tct resources to TIZEN device,it can be omitted")
    parser.add_option(
        "--purge", dest="purge", action="store_true", default=False,
        help="Clean up tct resources from TIZEN device")
    parser.add_option(
        "--check", dest="check", action="store_true", default=False,
        help="Check test resource on device")
    parser.add_option(
        "--deviceid", dest="deviceid", action="store",
        help="Set the device serial number")
    parser.add_option(
        "--procid", dest="process", action="store",
        help="Set the process id for checking")
    parser.add_option(
        "--bhtest", dest="bhtest", action="store",
        help="Install behavior test tool to device")

    (options, args) = parser.parse_args()

    if options.deviceid:
        SDB_SERNO += " -s " + options.deviceid

    options.install = True
    sdb_root_on()
    if not get_device_arch():
        sys.exit(0)        

    if options.purge:
        print "----------------------------------------------"
        print "[ Uninstall test resource on device. Please wait...]"
        uninstall_testconfig()
        uninstall_behavior()
        #uninstall_widget('getCap')
        kill_process("tinyweb")
        remove_tinyweb()
        kill_process("testkit-stub")
        remove_stub()
        print "Clean the tct packages in device successfully."
        print "----------------------------------------------"
        sys.exit(0)

    elif options.check:
        check_resource_dir()
        print "------------------------------------------------------------------------"
        print "[ Check test resource on device. Please wait...]"
        if options.process == 'tinyweb':
            check_tinyweb()
        elif options.process == 'testkit-stub':
            check_stub()
        elif options.process == 'testconfig':
            check_testconfig()
        elif options.process == 'getCap':
            check_getCap()
        else:
            check_tinyweb()
            check_stub()
            check_testconfig()
            check_getCap()

        print "Check finished."
        sys.exit(0)

    elif options.bhtest:
        print "------------------------------------------------------------------------"
        uninstall_behavior()
        print "[ Install tct behavior tool on device. Please wait...]"
        install_behavior(options.bhtest)
        print "Installation of tct behavior on device completed."
        sys.exit(0)
    elif options.install:
        check_resource_dir()
        print "------------------------------------------------------------------------"
        print "[ Clean old test resource on device. Please wait...]"
        uninstall_testconfig()
        #uninstall_widget("getCap")
        kill_process("tinyweb")
        remove_tinyweb()
        kill_process("testkit-stub")
        remove_stub()
        print "Clean the tct packages in device successfully."

        print "------------------------------------------------------------------------"
        print "[ Install test resource on device. Please wait...]"
        install_testconfig()
        install_stub()
        install_tinyweb()
        launch_tinyweb()
        install_widget("getCap")
        launch_widget("getCap")
        time.sleep(3)
        print "All of installation process on device completed."
        sys.exit(0)
